import chromeP from 'webext-polyfill-kinda';
import { getManifestPermissions } from 'webext-additional-permissions';
const contextMenuId = 'webext-domain-permission-toggle:add-permission';
let currentTabId;
let globalOptions;
async function executeCode(tabId, function_, ...args) {
    return chromeP.tabs.executeScript(tabId, {
        code: `(${function_.toString()})(...${JSON.stringify(args)})`
    });
}
async function isOriginPermanentlyAllowed(origin) {
    return chromeP.permissions.contains({
        origins: [origin + '/*']
    });
}
function createMenu() {
    var _a;
    const optionalHosts = (_a = chrome.runtime.getManifest()
        .optional_permissions) === null || _a === void 0 ? void 0 : _a.filter(permission => permission.includes('*') || permission === '<all_urls>');
    if (!optionalHosts || optionalHosts.length === 0) {
        throw new TypeError('Only permissions specified in the manifest may be requested. The manifest doesn’t specify any hosts in `optional_permissions`');
    }
    chrome.contextMenus.remove(contextMenuId, () => chrome.runtime.lastError);
    chrome.contextMenus.create({
        id: contextMenuId,
        type: 'checkbox',
        checked: false,
        title: globalOptions.title,
        contexts: ['page_action', 'browser_action'],
        // Note: This is completely ignored by Chrome and Safari. Great.
        documentUrlPatterns: optionalHosts
    });
}
function updateItem({ tabId }) {
    chrome.tabs.executeScript(tabId, {
        code: 'location.origin'
    }, async ([origin] = []) => {
        const settings = {
            checked: false,
            enabled: true
        };
        if (!chrome.runtime.lastError && origin) {
            // Manifest permissions can't be removed; this disables the toggle on those domains
            const manifestPermissions = await getManifestPermissions();
            const isDefault = manifestPermissions.origins.some(permission => permission.startsWith(origin));
            settings.enabled = !isDefault;
            // We might have temporary permission as part of `activeTab`, so it needs to be properly checked
            settings.checked = isDefault || await isOriginPermanentlyAllowed(origin);
        }
        chrome.contextMenus.update(contextMenuId, settings);
    });
}
async function togglePermission(tab, toggle) {
    // Don't use non-ASCII characters because Safari breaks the encoding in executeScript.code
    const safariError = 'The browser didn\'t supply any information about the active tab.';
    if (!tab.url && toggle) {
        throw new Error(`Please try again. ${safariError}`);
    }
    if (!tab.url && !toggle) {
        throw new Error(`Couldn't disable the extension on the current tab. ${safariError}`);
    }
    // TODO: Ensure that URL is in `optional_permissions`
    const permissionData = {
        origins: [
            new URL(tab.url).origin + '/*'
        ]
    };
    if (!toggle) {
        void chromeP.permissions.remove(permissionData);
        return;
    }
    const userAccepted = await chromeP.permissions.request(permissionData);
    if (!userAccepted) {
        chrome.contextMenus.update(contextMenuId, {
            checked: false
        });
        return;
    }
    if (globalOptions.reloadOnSuccess) {
        void executeCode(tab.id, (message) => {
            if (confirm(message)) {
                location.reload();
            }
        }, globalOptions.reloadOnSuccess);
    }
}
async function handleClick({ checked, menuItemId }, tab) {
    if (menuItemId !== contextMenuId) {
        return;
    }
    try {
        await togglePermission(tab, checked);
    }
    catch (error) {
        if (tab === null || tab === void 0 ? void 0 : tab.id) {
            executeCode(tab.id, 'alert' /* Can't pass a raw native function */, String(error instanceof Error ? error : new Error(error.message))).catch(() => {
                alert(error); // One last attempt
            });
            updateItem({ tabId: tab.id });
        }
        throw error;
    }
}
/**
 * Adds an item to the browser action icon's context menu.
 * The user can access this menu by right clicking the icon. If your extension doesn't have any action or
 * popup assigned to the icon, it will also appear with a left click.
 *
 * @param options {Options}
 */
export default function addDomainPermissionToggle(options) {
    if (globalOptions) {
        throw new Error('webext-domain-permission-toggle can only be initialized once');
    }
    const { name } = chrome.runtime.getManifest();
    globalOptions = {
        title: `Enable ${name} on this domain`,
        reloadOnSuccess: `Do you want to reload this page to apply ${name}?`,
        ...options
    };
    chrome.contextMenus.onClicked.addListener(handleClick);
    chrome.tabs.onActivated.addListener(updateItem);
    chrome.tabs.onUpdated.addListener((tabId, { status }) => {
        if (currentTabId === tabId && status === 'complete') {
            updateItem({ tabId });
        }
    });
    createMenu();
}
